<?php

namespace Controllers;

// use Teknisa\GER\Utilities\Environment;
// use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
// use Zeedhi\Framework\DTO\Response\Message;

class FrequenciaCicloAnual extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName = 'frciclocia_anual';
    protected $parameterBagColumns = array("DTINICIAL", "DTFINAL", "NRORG", "CDFILIAL", "CDCIA");
	protected $connection;


    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $connection) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection = $connection;
    }

    public function save(DTO\Request $request, DTO\Response $response){
    	$this->dataSourceName = 'frciclocia';
    	$this->parameterBagColumns = array();
		$rows = array();
		$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
		$dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();

		foreach ($request->getDataSet()->getRows() as $row) {
	        $condition = array(
	            array(
	                "value" => $row['CDFILIAL'],
	                "operator" => "=",
	                "columnName" => "CDFILIAL"
	            ),
	            array(
	                "value" => $row['CDCIA'],
	                "operator" => "=",
	                "columnName" => "CDCIA"
	            ),
	            array(
	            	"value" => $row['DTCICLOCIA'],
	                "operator" => "=",
	                "columnName" => "DTCICLOCIA"
	            ),
	            array(
	                "value" => $row['NRORG'],
	                "operator" => "=",
	                "columnName" => "NRORG"
	            )

	        );
	        $filterCriteria = new FilterCriteria($this->dataSourceName, $condition);
	        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
	        $newdataSet = $this->dataSourceManager->findBy($newFilterCriteria);
	        $newRows = $newdataSet->getRows();

	        if(!empty($newRows)){
	        	$row['__is_new'] = false;
	        }
	    }
    	parent::save($request, $response);
    }

    public function deleteCiclo(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'frciclocia_cdciclo';
        try {
            $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $this->crudService->delete($response, $dataSet);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 500));
        }
    }
    
    public function importarFrequencia(DTO\Request $request, DTO\Response $response){
    	$dataSource = 'frciclocia';
    	try{
    		$this->connection->beginTransaction();
	    	$row = $request->getRow();
	    	
			$frequenciaNoPeriodo = $this->buscaFrequenciasNoPeriodo($row, $row['CDCIA']);
			
			$rowsASeremSalvas = $this->preparaRows($frequenciaNoPeriodo, $row['CDCIADESTINO']);
			$this->apagaFrequenciaNoPeriodo($row, $row['CDCIADESTINO']);
			
			$this->vinculaCiclos($rowsASeremSalvas, $row, $row['CDCIA']);
			
			$newDataSet = new DataSet($dataSource, $rowsASeremSalvas);
			$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
			$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSource);
			$dataSet = new DataSet($dataSource, $rowsToSave);
			$this->dataSourceManager->persist($dataSet);
			$this->connection->commit();
    	} catch(\Exception $e){
    		$this->connection->rollback();
    		$response->setError(new Error($e->getMessage(), 500));
    	}
		
    }
    
    public function exportarFrequencia(DTO\Request $request, DTO\Response $response){
    	$dataSource = 'frciclocia';
    	try{
    		$this->connection->beginTransaction();
			$row = $request->getRow();
			
			$frequenciaNoPeriodo = $this->buscaFrequenciasNoPeriodo($row, $row['CDCIAORIGEM']);
			
			$rowsASeremSalvas = $this->preparaRows($frequenciaNoPeriodo, $row['CDCIA']);
			
			$this->apagaFrequenciaNoPeriodo($row, $row['CDCIA']);
			
			$this->vinculaCiclos($rowsASeremSalvas, $row, $row['CDCIAORIGEM']);
			$newDataSet = new DataSet($dataSource, $rowsASeremSalvas);
			$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
			$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSource);
			$dataSet = new DataSet($dataSource, $rowsToSave);
			$this->dataSourceManager->persist($dataSet);
			$this->connection->commit();
		} catch(\Exception $e){
			$this->connection->rollback();
    		$response->setError(new Error($e->getMessage(), 500));
    	}
    }
    
    private function buscaFrequenciasNoPeriodo($row, $companhia){
    	
    	$query = 'SELECT CDFILIAL,
                         CDCIA,
                         CDCICLO,
                         DTCICLOCIA,
                         NRORG
                    FROM FRCICLOCIA
                   WHERE CDFILIAL = :CDFILIAL
                     AND CDCIA = :CDCIA
                     AND DTCICLOCIA BETWEEN :DTINICIAL AND :DTFINAL
                     AND NRORG = :NRORG';
		$stmt = $this->connection->prepare($query);
		$params = array(
			'CDFILIAL'  => $row['CDFILIAL'],
			'CDCIA'     => $companhia,
			'DTINICIAL' => $row['DTINICIAL'],
			'DTFINAL'   => $row['DTFINAL'],
			'NRORG' => $this->environment->getNrOrgTrab()
		);
		$stmt->execute($params);
		$result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
		
		return $result;
    	
    }
    
    private function apagaFrequenciaNoPeriodo($row, $cdcompanhia){
    	
    	$query = 'DELETE FROM FRCICLOCIA
                   WHERE CDFILIAL = :CDFILIAL
                     AND CDCIA = :CDCIA
                     AND DTCICLOCIA BETWEEN :DTINICIAL AND :DTFINAL
                     AND NRORG = :NRORG';
                     
		$stmt = $this->connection->prepare($query);
		
		$params = array(
			'CDFILIAL'  => $row['CDFILIAL'],
			'CDCIA'     => $cdcompanhia,
			'DTINICIAL' => $row['DTINICIAL'],
			'DTFINAL'   => $row['DTFINAL'],
			'NRORG' => $this->environment->getNrOrgTrab()
		);
		
		$stmt->execute($params);
		
    }
    
    private function preparaRows($rowsImportadas, $cdcompanhiaDestino){
    	
		foreach ($rowsImportadas as &$item) {
			$item['CDCIA'] = $cdcompanhiaDestino;
			$item['changed'] = false;
			$item['__is_new'] = true;
		}
		
		return $rowsImportadas;
	
    }
    
    private function vinculaCiclos($rowsASeremSalvas, $row, $cdcompanhia){
    	
    	$ciclosNaoVinculados = $this->buscaCiclosNaoVinculados($rowsASeremSalvas, $row);
    	
    	$this->vincularCiclosACompanhiaDestino($ciclosNaoVinculados, $cdcompanhia);
    	
    }
    
    private function buscaCiclosNaoVinculados($rowsASeremSalvas, $row){
    	
    	$ciclosNaoVinculados = array();
    	
    	foreach($rowsASeremSalvas as $dia){
    		
			$query = 'SELECT CDFILIAL,
                             CDCIA,
                             CDCICLO,
                             NRORG
                        FROM CICLOCIA
                       WHERE CDFILIAL = :CDFILIAL
                         AND CDCIA = :CDCIA
                         AND CDCICLO = :CDCICLO
                         AND NRORG = :NRORG';
                         
			$stmt = $this->connection->prepare($query);
			
			$params = array(
				'CDFILIAL'  => $dia['CDFILIAL'],
				'CDCIA'     => $dia['CDCIA'],
				'CDCICLO'   => $dia['CDCICLO'],
				'NRORG'     => $dia['NRORG']
			);
			
			$stmt->execute($params);
			
			$result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
    		if(empty($result)){
    			array_push($ciclosNaoVinculados, array(
    				'CDFILIAL'  => $dia['CDFILIAL'],
					'CDCIA'     => $dia['CDCIA'],
					'CDCICLO'   => $dia['CDCICLO'],
					'NRORG'     => $dia['NRORG'],
					'__is_new'  => true
    			));
    		}
    	}
    	
		return array_unique($ciclosNaoVinculados, SORT_REGULAR);
		
    }
    
    private function vincularCiclosACompanhiaDestino($ciclosNaoVinculados, $cdcompanhia){
    	
    	foreach($ciclosNaoVinculados as &$ciclo){
    		
    		$filtro = array(
				array(
						"columnName" => "CDFILIAL",
						"operator" => "=",
						"value" => $ciclo['CDFILIAL']
				),
				array(
						"columnName" => "CDCIA",
						"operator" => "=",
						"value" => $cdcompanhia
				),
				array(
						"columnName" => "CDCICLO",
						"operator" => "=",
						"value" => $ciclo['CDCICLO']
				),
				array(
						"columnName" => "NRORG",
						"operator" => "=",
						"value" => $ciclo['NRORG']
				)
			);
	
			$filterCriteria = new FilterCriteria('ciclocia', $filtro);
			$filterCriteria =  $this->crudService->populateParameterBag($filterCriteria, array());
			$dataSet = $this->dataSourceManager->findBy($filterCriteria);
			$registros = $dataSet->getRows();
			
			$ciclo['NRCORCICCIA'] = $registros[0]['NRCORCICCIA'];
			
    	}
		$newDataSet = new DataSet('ciclocia', $ciclosNaoVinculados);
		$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
		$rowsToSave = $this->crudService->getRowsToSave($newRequest, 'ciclocia');
		$dataSet = new DataSet('ciclocia', $rowsToSave);
		$this->dataSourceManager->persist($dataSet);
    	
    }
    
}