<?php

namespace Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DTO\Response\Error;

Class ManutencaoCustoProdutoController extends \Teknisa\GER\Controllers\TeknisaCrud {

	const PADRAO = 'P';
	const COMPANHIA = 'C';

    protected $connection;
    protected $now;
	protected $environment;

	public function __construct($crudService, $parameterBag, $connection, $environment) {
        $this->connection = $connection;
        $this->environment = $environment;
        $this->now = new \DateTime();
        parent::__construct($crudService, $parameterBag);
	}

	public function salvaManutencaoProduto(DTO\Request $request, DTO\Response $response) {

        $produtos = $request->getDataSet()->getRows();

        $errors = array();

        for($i=0; $i < count($produtos); $i++) {

            try {

                $produto = $produtos[$i];
                $idcuspad = $produto['IDCUSPAD'];

                if($idcuspad === self::PADRAO)
                    $this->salvaManutencaoProdutoap($produto);

                if($idcuspad === self::COMPANHIA)
                    $this->salvaManutencaoProdutoApCia($produto);

            }catch(\Exception $e) {
                $errors[] = array(
                    'message' => "COC03300_MANUT_N_SALVA#_#".$produto['NMPRODUTO']
                );
            }
        }

        if (count($errors) > 0) {
            $response->addDataSet(new DataSet('errors', $errors));
            $response->setError(new Error('Produtos não salvo', 0));
        }

	}

    // temporary
	private function salvaManutencaoProdutoap($produto) {
        $sql = "UPDATE PRODUTOAP
                   SET VRMEDLIQAP = :VRMEDLIQAP,
                       VRMEDBRUAP = :VRMEDBRUAP,
                       VRULTCLIQAP = :VRULTCLIQAP,
                       VRULTCBRUAP = :VRULTCBRUAP,
                       DTULTATU = :NOW,
                       DTULTATUPRAP = :NOW,
                       CDOPERULTATU = :CDOPERULTATU,
                       NRORGULTATU = :NRORGULTATU
                 WHERE CDFILIAL = :CDFILIAL
                   AND CDPRODUTO = :CDPRODUTO
                   AND DTCUSTOAP = :DTCUSTOAP
                   AND NRORG = :NRORG";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(array(
            "VRMEDLIQAP" => $produto["VRMEDLIQAP"],
            "VRMEDBRUAP" => $produto["VRMEDBRUAP"],
            "VRULTCLIQAP" => $produto["VRULTCLIQAP"],
            "VRULTCBRUAP" => $produto["VRULTCBRUAP"],
            "CDFILIAL" => $produto["CDFILIAL"],
            "CDPRODUTO" => $produto["CDPRODUTO"],
            "DTCUSTOAP" => $produto['DTCUSTOAP'],
            "NRORG" => $produto['NRORG'],
            "CDOPERULTATU" => $this->environment->getCdOperador(),
            "NRORGULTATU" => $this->environment->getNrOrgTrab(),
            "NOW" => $this->now->format('dmY')
        ));

        $count = $stmt->rowCount();
        if($count === 0) {
            throw new \Exception("erro", 1);
        }

    }

	private function salvaManutencaoProdutoApCia($produto) {

        $sql = "UPDATE PRODUTOAPCIA
                   SET VRMEDLIQAP = :VRMEDLIQAP,
                       VRMEDBRUAP = :VRMEDBRUAP,
                       VRULTCLIQAP = :VRULTCLIQAP,
                       VRULTCBRUAP = :VRULTCBRUAP,
                       DTULTATU = :NOW,
                       CDOPERULTATU = :CDOPERULTATU,
                       NRORGULTATU = :NRORGULTATU
                 WHERE CDFILIAL = :CDFILIAL
                   AND CDCIA = :CDCIA
                   AND CDPRODUTO = :CDPRODUTO
                   AND DTCUSTOAP = :DTCUSTOAP
                   AND NRORG = :NRORG";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(array(
            "VRMEDLIQAP" => $produto["VRMEDLIQAP"],
            "VRMEDBRUAP" => $produto["VRMEDBRUAP"],
            "VRULTCLIQAP" => $produto["VRULTCLIQAP"],
            "VRULTCBRUAP" => $produto["VRULTCBRUAP"],
            "CDFILIAL" => $produto["CDFILIAL"],
            "CDPRODUTO" => $produto["CDPRODUTO"],
            "CDCIA" => $produto["CDCIA"],
            "DTCUSTOAP" => $produto['DTCUSTOAP'],
            "NRORG" => $produto['NRORG'],
            "CDOPERULTATU" => $this->environment->getCdOperador(),
            "NRORGULTATU" => $this->environment->getNrOrgTrab(),
            "NOW" => $this->now->format('dmY')
        ));
        $count = $stmt->rowCount();
        if($count === 0) {
            throw new \Exception("erro", 1);
        }

    }

}