<?php

namespace Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response;
use \Zeedhi\Framework\DTO\Response\Error;

class PersonalizacaoCompanhia extends \Teknisa\GER\Controllers\TeknisaCrud{
    
    protected $dataSourceName = 'ciafili';
    protected $connection;
    
    public function __construct($crudservice, $parameterbag, $connection){
        $this->connection = $connection;
        parent::__construct($crudservice, $parameterbag);
    }
    
    public function salvaCompanhia(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        if($row['IDATIVO'] === 'N'){
            try{
                $this->connection->beginTransaction();
                // $this->inativaVooTrecho($request, $response);
                $this->inativaVoo($request, $response);
                $this->connection->commit();
            } catch(\Exception $e){
                $response->setError(new Error($e->getMessage(), 500));
                $this->connection->rollback();
            }
        }
        parent::save($request, $response);
    }
    
    private function inativaVooTrecho($request, $response){
        $row = $request->getRow();
        $params = array(
            "CDFILIAL" => $row['CDFILIAL'],
            "CDCIA" => $row['CDCIA'],
            "NRORG" => $row['NRORG']
        );
        if($this->verificaVooTrechoExistente($params)){
            $sql = "UPDATE VOOTRECHO
                       SET IDSITVOOTRE = '2'
                     WHERE CDFILIAL = :CDFILIAL 
                       AND CDCIA = :CDCIA 
                       AND NRORG = :NRORG";
            $stmt = $this->connection->prepare($sql);
            $stmt->execute($params);
        }
    }
    
    private function verificaVooTrechoExistente($params){
        $sql = "SELECT NRVOO, 
                       CDTRECHO 
                  FROM VOOTRECHO 
                 WHERE CDFILIAL = :CDFILIAL 
                   AND CDCIA = :CDCIA 
                   AND NRORG = :NRORG";
        $result = $this->connection->fetchAll($sql, $params);
        if(!empty($result)){
            return true;
        }else{
            return false;
        }
    } 
    
    private function inativaVoo($request, $response){
        $row = $request->getRow();
        $params = array(
            "CDFILIAL" => $row['CDFILIAL'],
            "CDCIA" => $row['CDCIA'],
            "NRORG" => $row['NRORG']
        );
        if($this->verificaVooExistente($params)){
            $sql = "UPDATE VOO
                       SET IDSITVOO = '2'
                     WHERE CDFILIAL = :CDFILIAL 
                       AND CDCIA = :CDCIA 
                       AND NRORG = :NRORG";
            $stmt = $this->connection->prepare($sql);
            $stmt->execute($params);
        }
    }
    
    private function verificaVooExistente($params){
        $sql = "SELECT NRVOO 
                  FROM VOO 
                 WHERE CDFILIAL = :CDFILIAL 
                   AND CDCIA = :CDCIA 
                   AND NRORG = :NRORG";
        $result = $this->connection->fetchAll($sql, $params);
        if(!empty($result)){
            return true;
        }else{
            return false;
        }
    }
    
}